/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Original Code is Martin Kutschker's code.
 *
 * The Initial Developer of the Original Code is Martin Kutschker.
 *
 * Copyright (c) 2001-2002 Martin Kutschker. All rights reserved.
 *
 * Contributor(s): 
 *   Martin Kutschker <Martin.T.Kutschker@blackbox.net> (Original Author)
 */

// clipboard XPConnect objects
var clip                   = null; // system clipboard object
var trans                  = null; // transferable object
var flavourArray           = null; // style flavours
var suppString             = null; // helper
var suppWString            = null; // helper
var checkPasteValueOnFocus = true;

window.addEventListener("load", initClipboard, false);

// set up UI
function initClipboard()
{
    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
      //avoid annoying popups....
      //user may have purposefully disabled it
      //alert("No XPConnect privileges - clipboard not available");
       return;
    }

    clip = Components.classes["@mozilla.org/widget/clipboard;1"]
            .createInstance(Components.interfaces.nsIClipboard);
   
    trans = Components.classes["@mozilla.org/widget/transferable;1"]
             .createInstance(Components.interfaces.nsITransferable);
    if (trans)
        trans.addDataFlavor("text/unicode");

    suppString = Components.classes["@mozilla.org/supports-string;1"]
                  .createInstance(Components.interfaces.nsISupportsString);
    if (suppString)
        suppString.data = "text/unicode";

    suppWString = Components.classes["@mozilla.org/supports-wstring;1"]
                   .createInstance(Components.interfaces.nsISupportsWString);

    flavourArray = Components.classes["@mozilla.org/supports-array;1"]
                    .createInstance(Components.interfaces.nsISupportsArray);
    if (suppString && flavourArray)
        flavourArray.AppendElement(suppString);

    if (clip && trans) {
        document.getElementById("copyCmd").setAttribute("disabled", "false");

        window.addEventListener("focus", checkPasteValue, false);
    }
}

// copy current value to clipboard
function copyValue()
{
    if (!clip || !trans || !suppWString)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var copytext = document.getElementById("display").getAttribute("value");

    suppWString.data = copytext;

    trans.setTransferData("text/unicode",suppWString,copytext.length*2);

    clip.emptyClipboard(clip.kGlobalClipboard);
    clip.setData(trans, null, clip.kGlobalClipboard);
}

// paste value from clipboard
function pasteValue()
{

    if (!clip || !trans)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    if (!checkPasteValueOnFocus && !checkPasteValue())
        return;

    var str       = new Object();
    var strLength = new Object();
    var pastetext = null;

    clip.getData(trans,clip.kGlobalClipboard);
    trans.getTransferData("text/unicode",str,strLength);

    if (str)
        str = str.value.QueryInterface(Components.interfaces.nsISupportsWString);
    if (str) 
        str = str.data.substring(0,strLength.value / 2);
    if (str) {
        if (isNaN(parseFloat(str)))
            alert(bundle.getString("invValAlert"));
        else
            dc.enterNumber(str);
    }
}

// test if paste value from clipboard (event handler)
function checkPasteValue(ev)
{
    if (!clip || !flavourArray)
        return false;

    const pc = document.getElementById("pasteCmd");

    try {
        /*
         * Need privileges to check clipboard, but cannot ask as
         * a "focus" loop might occur:
         * window gets focus only to loose it to the dialog,
         * regets the focus, etc.
         *
         */
        if (clip.hasDataMatchingFlavors (flavourArray, clip.kGlobalClipboard)) {
            if (!ev)
                return true;
            else
                pc.setAttribute("disabled", "false");
        }
        else {
            if (!ev)
                return false;
            else
                pc.setAttribute("disabled", "true");
        }
    }
    catch (err) {
        if (checkPasteValueOnFocus) {
            checkPasteValueOnFocus = false;

            window.removeEventListener("focus", checkPasteValue, false);

            // enable paste command permanently
            pc.setAttribute("disabled", "false");
        }
    }

    return true;
}
