/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Copyright (c) 2001 Martin Kutschker.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

function DeskCalculator(display, numClass, mathClass) /* extends Calculator */ {
  this.display     = display || null;
  this.displayRPN  = new Array(0);
  this.number      = numClass || null;
  this.math        = mathClass || null;
  this.value       = null; // unused
  this.newNumber   = true;
  this.hasFraction = false;
  this.inverse     = false;
  this.hyperbolic  = false;
  this.rpn         = false;

  // this.history = "";

  this.enterNumber       = enterNumber;
  this.typeNumeral       = typeNumeral;
  this.typeDecFract      = typeDecFract;
  this.typeOperator      = typeOperator;
  this.typeStdOperator   = typeStdOperator;
  this.typeRPNOperator   = typeRPNOperator;
  this.typeOperator      = typeOperator;
  this.typePowerOperator = typePowerOperator; // power of n or nth root
  this.typeEquate        = typeEquate;        // standard notation
  this.typeEnter         = typeEnter;         // reverse polish notation
  this.delNumeral        = delNumeral;
  this.toggleSign        = toggleSign;
  this.clear             = clear;
  this.clearAll          = clearAll;
  this.toggleNotation    = toggleNotation;

  this.setNumber      = setNumber;
  this.calcReciprocal = calcReciprocal;
  this.calcSquare     = calcSquare;
  this.calcSquareRoot = calcSquareRoot;
  this.calcPercentage = calcPercentage;
  this.calcFunction   = calcFunction;
  this.enterConstant  = enterConstant;

  /***********************************
   *                                 *
   * methods for handling user input *
   *                                 *
   ***********************************/

  // user inputs (eg pastes from clipboard) a number (eg: 12.3)
  function enterNumber(aNumber) {
    var value = "" + aNumber; // force string

    this.display.setAttribute("value", value);

    if (value == "0")
        this.newNumber = true;
    else
        this.newNumber = false;

    if (value.indexOf(".") != -1)
        this.hasFraction = true;
    else
        this.hasFraction = false;
  }

  // user types a single numeral (eg: 1)
  function typeNumeral(aNumeral) {
    if (this.newNumber) {
      this.display.setAttribute("value", aNumeral);
      if (aNumeral != "0")
        this.newNumber = false;
    }
    else
      this.display.setAttribute("value", this.display.getAttribute("value") + aNumeral);
  }

  // user types the decimal fraction point
  function typeDecFract() {
    if (!this.hasFraction) {
      if (this.newNumber)
        this.display.setAttribute("value", "0.");
      else 
        this.display.setAttribute("value", "" + this.display.getAttribute("value") + ".");

      this.newNumber   = false;
      this.hasFraction = true;
    }
  }

  // users deletes last character (numeral or fraction)
  function delNumeral() {
    var v = this.display.getAttribute("value");

    if (this.newNumber)
      return;

    if (this.hasFraction && v.substr(-1,1) == ".")
      this.hasFraction = false;

    v = v.substring(0,v.length-1);

    if (v == "") {
      v              = "0"
      this.newNumber = true;
    }

    this.display.setAttribute("value", v);
  }

  function typeOperator(aOp) {
    if (this.rpn)
      this.typeRPNOperator(aOp);
    else
      this.typeStdOperator(aOp);
  }

  // user switches between positive and negative display value
  function toggleSign() {
    var value = this.number.parse(this.display.getAttribute("value"));

    this.display.setAttribute("value", value.neg());
  }

  // user clears current number (display)
  function clear() {
    this.display.setAttribute("value", "0");

    this.newNumber   = true;
    this.hasFraction = false;
  }

  // user clears current equation (display and execution stack)
  function clearAll() {
    var i;

    this.display.setAttribute("value", "0");

    if (this.rpn) {
      for (i=0; i < this.displayRPN.length; i++) {
        this.displayRPN[i].setAttribute("value", "");
      }
    }

    this.newNumber   = true;
    this.hasFraction = false;

    this.reset();
  }

  function toggleNotation() {
    this.rpn = (this.rpn ? false : true);

    this.clearAll();
  }

  /*******************************************************
   *                                                     *
   * methods for handling user input (standard notation) *
   *                                                     *
   *******************************************************/

  // user types an operator (ends previous number input starting a new one)
  function typeStdOperator(iOp) {
    if (this.newNumber) {
      if (this.operators.length > 0) { // after previous equate
        this.operators[this.operators.length-1] = iOp; // replace current operator

        return;
      }
    }
    this.operands.push(this.number.parse(this.display.getAttribute("value")));

    // reduce stack
    while ((this.operators.length >= 1) &&
           (CALC_OPPRECED[this.operators[this.operators.length-1]] >=
            CALC_OPPRECED[iOp])
          ) {

/*
      // must go to Calculator object
      this.history = this.operands[this.operands.length-2] +
                     CALC_OPS[this.operators[this.operators.length-1]] +
                     ( this.history ? this.history :
                        this.operands[this.operands.length-1]);
*/

      this.calcTop();
      this.display.setAttribute("value", this.operands[this.operands.length-1]);
    }
    this.operators.push(iOp);

    this.newNumber   = true;
    this.hasFraction = false;
  }

  // user requests calculation of current equation
  function typeEquate() {
    if (this.rpn)
      return;

    // commenting this out takes current display value as additional operand
    // this gives a kind of "constant automatic" for the last operation
    // (uncommenting breaks percentage - operator will be discarded, but should not!)
/*
    if (this.newNumber) { // remove last (dangling) operator
      if (this.operators.length > 0)
        this.operators.pop();
    }
    else
*/
    this.operands.push(this.number.parse(this.display.getAttribute("value")));

    // sanity check
    if (this.operands.length < 2 || this.operators.length < 1) {
      this.newNumber   = true;
      this.hasFraction = false;

      this.reset();

      return;
    }

    this.calculate();

    // sanity check
    if (this.operands.length > 1 || this.operators.length > 0) {
      alert("error - exactly one operand expected: " +
            this.operands.length + " operand(s), " +
            this.operators.length + " operator(s)");

      this.clearAll();
    }
    else {
      var result = this.operands.pop();
      this.display.setAttribute("value", result);
    }

    this.newNumber   = true;
    this.hasFraction = false;
  }

  /********************************************************
   *                                                      *
   * methods for handling user input (rev. pol. notation) *
   *                                                      *
   ********************************************************/

  // user types an operator (ends previous number input calculating stack)
  function typeRPNOperator(aOp) {
    var i, n;

    if (this.operands.length == 0)
      return;

    this.operands.push(this.number.parse(this.display.getAttribute("value")));
    this.operators.push(aOp);
    dc.calcTop();

    // refresh stack display
    for (i=0, n=this.operands.length-2;
         i < this.displayRPN.length;
         i++, n--) {
      if (n >= 0)
        this.displayRPN[i].setAttribute("value", this.operands[n]);
      else
        this.displayRPN[i].setAttribute("value", "");
    }
    this.display.setAttribute("value", this.operands.pop());

    this.newNumber   = true;
    this.hasFraction = false;
  }

  function typeEnter() {
    var i, n;

    if (!this.rpn)
      return;

    this.operands.push(this.number.parse(this.display.getAttribute("value")));

    for (i=0, n=this.operands.length-1;
         i < this.displayRPN.length && n >= 0;
         i++, n--) {
      this.displayRPN[i].setAttribute("value", this.operands[n]);
    }

    this.display.setAttribute("value", "0");
    this.newNumber   = true;
    this.hasFraction = false;
  }

  /************************************
   *                                  *
   * common desk calculator functions *
   *                                  *
   ***********************************/

  function setNumber(aNumber) {
    display.setAttribute("value", aNumber);

    this.newNumber   = true;
    this.hasFraction = false;
  }

  function calcReciprocal() {
    var value = this.number.parse(this.display.getAttribute("value"));

    value = value.div(1, value);

    if (!isFinite(value))
      this.clearAll();

    this.setNumber(value);
  }

  function calcSquare() {
    var value = this.number.parse(this.display.getAttribute("value"));

    value = this.math.pow(value,2);

    this.setNumber(value);
  }

  function calcSquareRoot() {
    var value = this.number.parse(this.display.getAttribute("value"));

    value = this.math.sqrt(value);

    if (!isFinite(value))
      this.clearAll();

    this.setNumber(value);
  }

  function calcPercentage() {
    // sanity check
    if (this.newNumber)
      return;

    var value = this.number.parse(this.display.getAttribute("value"));

    if (this.rpn) {
      if (this.operands.length == 0)
        value.div(100);
      else {
        value.mul(this.number.div(this.operands[this.operands.length-1], 100));        
      }
    }
    else {
      if (this.operators.length == 0) // after equate
        value.div(100);
      else {
        // sanity check
        if (this.operands.length == 0)
          return;

        value.mul(this.number.div(this.operands[this.operands.length-1], 100));
      }
    }

    this.setNumber(value);
  }

  function calcFunction(func) {
    var value = this.number.parse(this.display.getAttribute("value"));

    if (this.inverse) {
      switch (func) {
        case "sin":
        case "cos":
        case "tan":
        case "sinh":
        case "cosh":
        case "tanh":
          func = "a" + func;
          break;

        case "ln":
          func = "exp";
          break;

        case "lg":
          value = this.math.pow(10,value);
          this.setNumber(value);
          return;

        case "ld":
          value = this.math.pow(2,value);
          this.setNumber(value);
          return;
      }
    }

    if (this.hyperbolic) {
      switch (func) {
        case "sin":
        case "cos":
        case "tan":
        case "asin":
        case "acos":
        case "atan":
          func = func + "h";
          break;
      }
    }

    value = this.evalFunction(func,value);

    this.setNumber(value);
  }

  function enterConstant(aConst) {
    aConst = aConst.toUpperCase();

    this.setNumber(this.math[aConst]);
  }

  function typePowerOperator() {
    if (this.inverse)
      this.typeOperator(CALC_ROT)
    else
      this.typeOperator(CALC_POW);
  }
}
DeskCalculator.prototype = new Calculator;
