/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Original Code is Martin Kutschker's code.
 *
 * The Initial Developer of the Original Code is Martin Kutschker.
 *
 * Copyright (c) 2001-2002 Martin Kutschker. All rights reserved.
 *
 * Contributor(s): 
 *   Martin Kutschker <Martin.T.Kutschker@blackbox.net> (Original Author)
 */

const PANELS_RDF_FILE  = "UPnls";
const DIR_SERV_CID     = "@mozilla.org/file/directory_service;1";
const STD_URL_CID      = "@mozilla.org/network/standard-url;1";
const RDF_CID          = "@mozilla.org/rdf/rdf-service;1";
const CONTAINER_CID    = "@mozilla.org/rdf/container;1";

var title = "mozCalc";
var URL = "chrome://mozcalc/content/mozcalcPanel.xul";
var nc = "http://home.netscape.com/NC-rdf#";
var resource = "urn:sidebar:current-panel-list";

// test if mozCalc already on sidebar panel
function checkPanel()
{
  const nsIRDFService = Components.interfaces.nsIRDFService;
  const nsIRDFRemoteDataSource = Components.interfaces.nsIRDFRemoteDataSource;

  var pami = document.getElementById("panelAddMenuItem");

  var rdf = Components.classes[RDF_CID].getService(nsIRDFService);
  var uri = getSidebarDatasourceURI(PANELS_RDF_FILE);
  var datasource = rdf.GetDataSource(uri);
  var container = getSidebarPanelsContainer(rdf, datasource);

  var panel_resource = rdf.GetResource("urn:sidebar:3rdparty-panel:" + URL);
  var panel_index = container.IndexOf(panel_resource);

  if (panel_index != -1)
    pami.setAttribute("disabled", "true");
  else
    pami.setAttribute("disabled", "false");
}

function addChromeToPanel() {
  const nsIRDFService = Components.interfaces.nsIRDFService;
  const nsIRDFRemoteDataSource = Components.interfaces.nsIRDFRemoteDataSource;

  var rdf = Components.classes[RDF_CID].getService(nsIRDFService);
  var uri = getSidebarDatasourceURI(PANELS_RDF_FILE);
  var datasource = rdf.GetDataSource(uri);
  var container = getSidebarPanelsContainer(rdf, datasource);

  /* Create a resource for the new panel and add it to the list */
  var panel_resource = rdf.GetResource("urn:sidebar:3rdparty-panel:" + URL);
  var panel_index = container.IndexOf(panel_resource);
  if (panel_index != -1) // panel is already in list
    return;

  container.AppendElement(panel_resource);

  /* Now make some sidebar-ish assertions about it... */
  datasource.Assert(panel_resource, rdf.GetResource(nc + "title"),
                                    rdf.GetLiteral(title), true);
  datasource.Assert(panel_resource, rdf.GetResource(nc + "content"),
                                    rdf.GetLiteral(URL), true);

  // Use an assertion to pass a "refresh" event to all the sidebars.
  // They use observers to watch for this assertion (in sidebarOverlay.js).
  datasource.Assert(rdf.GetResource(resource),
                    rdf.GetResource(nc + "refresh"),
                    rdf.GetLiteral("true"),
                          true);
  datasource.Unassert(rdf.GetResource(resource),
                      rdf.GetResource(nc + "refresh"),
                      rdf.GetLiteral("true"));

  /* Write the modified panels out. */
  datasource.QueryInterface(nsIRDFRemoteDataSource).Flush();
}

function getSidebarDatasourceURI(panels_file_id)
{
  try 
  {
    /* use the fileLocator to look in the profile directory 
     * to find 'panels.rdf', which is the
     * database of the user's currently selected panels. */
    var directory_service = Components.classes[DIR_SERV_CID].getService();
    if (directory_service)
      directory_service = directory_service.QueryInterface(Components.interfaces.nsIProperties);

    /* if <profile>/panels.rdf doesn't exist, get will copy
     * bin/defaults/profile/panels.rdf to <profile>/panels.rdf */
    var sidebar_file = directory_service.get(panels_file_id, Components.interfaces.nsIFile);

    if (!sidebar_file.exists())
    {
      /* this should not happen, as GetFileLocation() should copy
       * defaults/panels.rdf to the users profile directory */
      return null;
    }

    var file_url = Components.classes[STD_URL_CID].createInstance(Components.interfaces.nsIFileURL);
    file_url.file = sidebar_file;

    return file_url.spec;
  }
  catch (ex)
  {
    /* this should not happen */
    return null;
  }

  return null;
}

function getSidebarPanelsContainer(rdf, datasource) {
  const nsIRDFContainer  = Components.interfaces.nsIRDFContainer;

  // Create a "container" wrapper around the current panels to
  // manipulate the RDF:Seq more easily.
  var panel_list = datasource.
                       GetTarget(rdf.GetResource(resource),
                                 rdf.GetResource(nc + "panel-list"),
                                 true);
  if (panel_list)
  {
    panel_list.QueryInterface(Components.interfaces.nsIRDFResource);
  }
  else
  {
    // Datasource is busted.

    return null;
  }

  var container = Components.classes[CONTAINER_CID].createInstance(nsIRDFContainer);
  container.Init(datasource, panel_list);

  return container;
}
