/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Original Code is Martin Kutschker's code.
 *
 * The Initial Developer of the Original Code is Martin Kutschker.
 *
 * Copyright (c) 2001-2002 Martin Kutschker. All rights reserved.
 *
 * Contributor(s): 
 *   Martin Kutschker <Martin.T.Kutschker@blackbox.net> (Original Author)
 */

var gStorageRootRDF = null;   // root datasource for storage (read-only)
var gStorageRDF     = null;   // datasource for storage
var gHistoryCnt     = 0;      // number of history entries
var gHistoryMax     = 20;     // max. history (should be a pref)
var gStorageWait    = 100;    // load retry counter

function getInMemoryStorage()
{
    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    gStorageRDF = new RDF("", 
                          "urn:storage",
                          "http://mozcalc.mozdev.org/rdf#");
    gStorageRDF.addSeq("urn:storage:memory");
    gStorageRDF.addSeq("urn:storage:history");

    for (var i=0; i<10; i++)
    {
        var n = "urn:storage:memory:" + i;
        gStorageRDF.addNode(n);
        gStorageRDF.setAttribute(n, "Name", i+":");
        gStorageRDF.setAttribute(n, "Value", 0);
    }
}

function attachStore()
{
    if (!gStorageRootRDF || !gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var tmpl = null;

    if (!gStorageRootRDF.loaded && !gStorageRDF.loaded)
    {
        if (!gStorageWait)
        {
          //avoid annoying popups....
          //user may have purposefully disabled it
          //alert("RDF data sources not loaded - memory and history not available");

            return;
        }

        window.setTimeout("attachStore()", 600);
        gStorageWait--;

        return;
    }

    // in-memory datasource init hook
    loadStore();

    gHistoryCnt = gStorageRDF.countSeqSubNodes("urn:storage:history");

    // Add datasource to sidebar
    tmpl = document.getElementById("sidebarTree");
    if (tmpl) {
      tmpl.database.AddDataSource(gStorageRootRDF.dsource);
      tmpl.database.AddDataSource(gStorageRDF.dsource);
      tmpl.builder.rebuild();
    }

    // Add datasource to Edit/Memory menu
    tmpl = document.getElementById("memoryPopup");
    if (tmpl) {
      tmpl.database.AddDataSource(gStorageRDF.dsource);
      tmpl.builder.rebuild();
    }

    // Add datasource to Edit/History menu
    tmpl = document.getElementById("historyPopup");
    if (tmpl) {
      tmpl.database.AddDataSource(gStorageRDF.dsource);
      tmpl.builder.rebuild();
    }
}

function storeMemValue(aNum)
{
    if (!gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var value = getValue();

    gStorageRDF.setAttribute("urn:storage:memory:" + aNum, "Value", value);
}

function recallMemValue(aNum)
{
    if (!gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var value = gStorageRDF.getAttribute("urn:storage:memory:" + aNum, "Value");

    dc.enterNumber(value);
}

function addMemValue(aNum)
{
    if (!gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var nval  = gStorageRDF.getAttribute("urn:storage:memory:" + aNum, "Value");
    var oval  = getValue();
    var value = parseFloat(nval) + parseFloat(oval);

    gStorageRDF.setAttribute("urn:storage:memory:" + aNum, "Value", value);

    dc.newNumber   = true;
    dc.hasFraction = false;
}

function clearMemValues()
{
    if (!gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    for (var i=0; i<10; i++)
    {
        gStorageRDF.setAttribute("urn:storage:memory:" + i, "Value", 0);
    }
}

function addToHistory(aValue)
{
    if (!gStorageRDF)
        return;

    try {
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e) {
        return;
    }

    var d = new Date();
    var n = "urn:storage:history:"+d.getTime();

    gStorageRDF.addNode(n, 1);
    gStorageRDF.setAttribute(n, "Value", aValue);
    gHistoryCnt++;

    if (gHistoryCnt > gHistoryMax)
      gStorageRDF.removeNodeAt("urn:storage:history", gHistoryMax+1);

    // HACK for bug #87500

    // find history's tree item
    var tmpl = document.getElementById("sidebarTree");
    if (tmpl) {
      var label = bundle.getString("history")
      var rows = tmpl.getElementsByAttribute("label",label);
      var item = rows[0].parentNode.parentNode;

      // close and reopen it
      if (item.getAttribute("open") == "true") {
        item.setAttribute("open","false");
        item.setAttribute("open","true");
      }
    }
    // HACK END
}

